'''
 Generated by EZX Code Generator  
Date: 04/18/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api




class PairsSpreadOrder(EzxMsg, metaclass=EzxMsgMeta):


	def __init__(self, **kwargs):
		self.msgType : int = None
		self.myID : str = None
		self.routerOrderID : int = None
		self.userNum : int = None
		self.userName : str = None
		self.side : int = None
		self.price : float = None
		self.orderQty : float = None
		self.icebergQty : int = None
		self.fairValue : float = None
		self.priceOffset : float = None
		self.xSymbol : str = None
		self.ySymbol : str = None
		self.xMdSymbol : str = None
		self.yMdSymbol : str = None
		self.xText : str = None
		self.yText : str = None
		self.destination : str = None
		self.account : str = None
		self.locateReqd : str = None
		self.hangAllowance : int = None
		self.moveAllowance : float = None
		self.slipAllowance : float = None
		self.xSlipAllowance : float = None
		self.ySlipAllowance : float = None
		self.yPriceWeight : float = None
		self.yHedgeWeight : float = None
		self.initialScan : float = None
		self.hedgeScan : float = None
		self.replaceThrottleTime : float = None
		self.maxDistanceFromMarket : float = None
		self.orderDelayMs : int = None
		self.idSourceX : str = None
		self.securityIdX : str = None
		self.idSourceY : str = None
		self.securityIdY : str = None
		super(PairsSpreadOrder, self).__init__(**kwargs)
		self.msg_type = api.IserverMsgType.APPLICATION.value
		self.msg_subtype = api.IserverMsgSubType.PAIRS_SPREAD_ORDER.value

