'''
 Generated by EZX Code Generator  
Date: 04/18/22 
'''

from enum import Enum, unique

@unique
class IserverMsgType(Enum):
	DISCONNECTED = 6
	SYNC = 1
	COMMAND = 3
	SESSION = 5
	UNDEFINED = 0
	SYNC_COMPLETE = 4
	APPLICATION = 2



@unique
class IserverMsgSubType(Enum):
	CLIENT_SETTINGS_UPDATE = 146
	READ_GROUP_ACCOUNTS = 136
	PARENT_ORDER_REQUEST = 115
	ORDERINFO_REQUEST = 105
	BROADCAST_RESPONSE = 125
	REJECT = 111
	DKTRADE = 124
	PARENT_ORDER_RESPONSE = 116
	UPDATE_GROUP_SETTINGS = 142
	FORGOTPASSWORD_RESPONSE = 135
	EXECUTION = 103
	CHANGEPASSWORD = 132
	STATUS = 120
	REPLACE_REJECT = 118
	DELETE_GROUP = 141
	ADMIN_RESPONSE = 10
	UPDATE_POSITION = 144
	DESTINATION_STATUS = 127
	ORDERREQUESTLIST = 38
	QUOTE_ACCEPT = 155
	GET_ORDERS_RESPONSE = 96
	MOVE_ACCOUNT = 140
	FIXEDINCOMEMARKETDATAINCREMENTAL = 148
	ALLOCINFO_RESPONSE = 92
	EXECUTION_RESPONSE = 108
	TEST_CHILD_CLASS = 152
	ORDERINFO_RESPONSE = 106
	CHANGEPASSWORD_RESPONSE = 133
	EXECUTION_REQUEST = 107
	UPDATE_GROUP_NAME = 143
	MARKETDATASNAPSHOT = 36
	ADMIN_COMMAND = 126
	QUOTE = 128
	FORGOTPASSWORD = 134
	POSITION_RESPONSE = 145
	ALLOCATION_RESPONSE = 94
	PAIRS_SPREAD_ORDER_RESPONSE = 150
	CANCEL_ALL = 11
	CLIENT_ORDER_RESPONSE = 99
	HEARTBEAT = 122
	GET_ORDERS_REQUEST = 95
	LOGONRESPONSE = 110
	CANCEL_REJECT = 119
	ORDERS = 113
	ORDERRESPONSE = 112
	BROADCAST = 104
	MARKETDATAINCREMENTAL = 37
	CREDIT_USED_UPDATE = 147
	CANCEL = 117
	COMPANY_SETTINGS_UPDATE = 131
	EXECUTIONS = 114
	UNSPECIFIED = 0
	TEST_PARENT_CLASS = 151
	CREATE_GROUP_ACCOUNT = 138
	ORDER = 102
	OMSSETTINGS = 97
	LOGON = 100
	QUOTE_RESPONSE = 154
	PAIRS_SPREAD_ORDER = 149
	CLIENT_ORDER_REQUEST = 98
	QUOTE_REQUEST = 129
	ALLOCINFO_REQUEST = 91
	MARKETDATA_HEALTH = 153
	DELETE_CLIENT_ORDER = 130
	ALLOCATION_REQUEST = 93
	DELETE_ACCOUNT = 139
	REPLACE = 121
	LOGOUT = 101
	IOI = 39



