'''
 Generated by EZX Code Generator  
Date: 04/18/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api




class ExecutionRequest(EzxMsg, metaclass=EzxMsgMeta):


	def __init__(self, **kwargs):
		self.execTransType : int = None
		self.execType : int = None
		self.orderQty : float = None
		self.ordType : str = None
		self.exeTimeStamp : str = None
		self.price : float = None
		self.lastPrice : float = None
		self.lastShares : float = None
		self.routerOrderID : int = None
		self.ssOrderID : int = None
		self.ssExeID : int = None
		self.trader : str = None
		self.text : str = None
		self.userInfo : str = None
		super(ExecutionRequest, self).__init__(**kwargs)
		self.msg_type = api.IserverMsgType.APPLICATION.value
		self.msg_subtype = api.IserverMsgSubType.EXECUTION_REQUEST.value

