'''
 Generated by EZX Code Generator  
Date: 04/18/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api
from iserver.msgs.PairsSpreadOrder import PairsSpreadOrder




class PairsSpreadOrderResponse(PairsSpreadOrder, metaclass=EzxMsgMeta):


	def __init__(self, **kwargs):
		self.filledQty : int = None
		self.hungQty : int = None
		self.xFilledQty : int = None
		self.yFilledQty : int = None
		self.xAvgPrice : float = None
		self.yAvgPrice : float = None
		self.pairState : str = None
		self.state : str = None
		self.event : str = None
		self.cumQty : float = None
		self.leavesQty : float = None
		self.timeStamp : str = None
		self.text : str = None
		self.ownerID : str = None
		super(PairsSpreadOrderResponse, self).__init__(**kwargs)
		self.msg_type = api.IserverMsgType.APPLICATION.value
		self.msg_subtype = api.IserverMsgSubType.PAIRS_SPREAD_ORDER_RESPONSE.value

