'''
 Generated by EZX Code Generator  
Date: 04/18/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api




class ExecutionStatus(EzxMsg, metaclass=EzxMsgMeta):


	def __init__(self, **kwargs):
		self.event : str = None
		self.execTransType : int = None
		self.execType : int = None
		self.orderQty : float = None
		self.ordType : str = None
		self.exeTimeStamp : str = None
		self.exeVersion : int = None
		self.price : float = None
		self.lastPrice : float = None
		self.lastShares : float = None
		self.options : str = None
		self.senderSubID : str = None
		self.ssExeID : int = None
		self.tagValueBlock : str = None
		self.targetSubID : str = None
		self.text : str = None
		self.transactTime : str = None
		self.tradeLiquidityIndicator : str = None
		self.ecnAccessFee : float = None
		self.cfiCode : str = None
		self.refID : str = None
		self.ratioQty : int = None
		self.positionEffect : str = None
		self.lastMkt : str = None
		self.iServerGenerated : int = None
		self.clientOrderID : str = None
		self.origClientOrderID : str = None
		self.brokerOrderID : str = None
		super(ExecutionStatus, self).__init__(**kwargs)

