'''
 Generated by EZX Code Generator  
Date: 04/18/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api

from .TagValueMsg import TagValueMsg



class Properties(EzxMsg, metaclass=EzxMsgMeta):

	__block_fields__ = {
		'STRATSETTVS' : Field('propertyMap', 'STRATSETTVS', noop_decode_func, ), 
	}

	def __init__(self, **kwargs):
		self.companyName : str = None
		self.propertyMap : TagValueMsg = None
		super(Properties, self).__init__(**kwargs)

