'''
 Generated by EZX Code Generator  
Date: 04/18/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api

from .YieldData import YieldData
from .PartiesStatus import PartiesStatus
from .BondExecutionStatus import BondExecutionStatus



class QuoteResponse(EzxMsg, metaclass=EzxMsgMeta):

	__block_fields__ = {
		'YDB' : Field('yieldData', 'YDB', noop_decode_func, ), 
		'PRTS' : Field('parties', 'PRTS', noop_decode_func, update_list_item), 
		'BEB' : Field('bondExecutionStatus', 'BEB', noop_decode_func, ), 
	}

	def __init__(self, **kwargs):
		self.quoteID : str = None
		self.quoteEntryID : str = None
		self.quoteRespID : str = None
		self.seqNo : int = None
		self.account : str = None
		self.clientOrderID : str = None
		self.state : str = None
		self.price : float = None
		self.side : int = None
		self.text : str = None
		self.settlementType : int = None
		self.cumQty : float = None
		self.avgPrice : float = None
		self.ordType : str = None
		self.orderQty : float = None
		self.minQty : float = None
		self.maxFloor : float = None
		self.quoteCondition : str = None
		self.numCompetitors : int = None
		self.coverPrice : float = None
		self.quoteRank : str = None
		self.yieldData : YieldData = None
		self.parties : list[PartiesStatus] = None
		self.bondExecutionStatus : BondExecutionStatus = None
		super(QuoteResponse, self).__init__(**kwargs)
		self.msg_type = api.IserverMsgType.APPLICATION.value
		self.msg_subtype = api.IserverMsgSubType.QUOTE_RESPONSE.value

