'''
 Generated by EZX Code Generator  
Date: 04/18/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api



# constants
STATE_DOWN = 'ENGINE_DOWN'
STATE_UP = 'ENGINE_UP'


class EngineStatus(EzxMsg, metaclass=EzxMsgMeta):


	def __init__(self, **kwargs):
		self.fixEngineName : str = None
		self.fixEngineState : str = None
		super(EngineStatus, self).__init__(**kwargs)
		self.msg_type = api.IserverMsgType.APPLICATION.value
		self.msg_subtype = api.IserverMsgSubType.STATUS.value

