'''
 Generated by EZX Code Generator  
Date: 04/18/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api

from .OrderRequestInfo import OrderRequestInfo



class OrderRequestList(EzxMsg, metaclass=EzxMsgMeta):

	__block_fields__ = {
		'ORB' : Field('orderList', 'ORB', noop_decode_func, update_list_item), 
	}

	def __init__(self, **kwargs):
		self.orderList : list[OrderRequestInfo] = None
		super(OrderRequestList, self).__init__(**kwargs)
		self.msg_type = api.IserverMsgType.APPLICATION.value
		self.msg_subtype = api.IserverMsgSubType.ORDERREQUESTLIST.value

