'''
 Generated by EZX Code Generator  
Date: 04/18/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api




class QuoteRecord(EzxMsg, metaclass=EzxMsgMeta):


	def __init__(self, **kwargs):
		self.updateType : int = None
		self.quoteType : int = None
		self.symbol : str = None
		self.quoteSize : float = None
		self.price : float = None
		self.quoteID : str = None
		self.quoteSource : str = None
		self.quoteDateTime : str = None
		self.settleDate : str = None
		super(QuoteRecord, self).__init__(**kwargs)

