'''
 Generated by EZX Code Generator  
Date: 04/18/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api

from .ChoiceGroup import ChoiceGroup
from .UpDown import UpDown
from .OfficeGroupClient import OfficeGroupClient
from .TraderList import TraderList



class Settings(EzxMsg, metaclass=EzxMsgMeta):

	__block_fields__ = {
		'CHOGBLK' : Field('choiceGroups', 'CHOGBLK', noop_decode_func, update_list_item), 
		'UPDWNBLK' : Field('updowns', 'UPDWNBLK', noop_decode_func, update_list_item), 
		'OGCBLK' : Field('officeGroupClients', 'OGCBLK', noop_decode_func, update_list_item), 
		'TRLST' : Field('tradersList', 'TRLST', noop_decode_func, update_list_item), 
	}

	def __init__(self, **kwargs):
		self.state : str = None
		self.gmtEndOfDay : str = None
		self.choiceGroups : list[ChoiceGroup] = None
		self.updowns : list[UpDown] = None
		self.officeGroupClients : list[OfficeGroupClient] = None
		self.tradersList : list[TraderList] = None
		super(Settings, self).__init__(**kwargs)

