'''
 Generated by EZX Code Generator  
Date: 04/18/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api

from .Properties import Properties



class CompanySettingsUpdate(EzxMsg, metaclass=EzxMsgMeta):

	__block_fields__ = {
		'PRPSBLK' : Field('settings', 'PRPSBLK', noop_decode_func, ), 
	}

	def __init__(self, **kwargs):
		self.settings : Properties = None
		self.notes : str = None
		super(CompanySettingsUpdate, self).__init__(**kwargs)
		self.msg_type = api.IserverMsgType.APPLICATION.value
		self.msg_subtype = api.IserverMsgSubType.COMPANY_SETTINGS_UPDATE.value

