'''
 Generated by EZX Code Generator  
Date: 04/18/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api
from iserver.msgs.ClientOrderRequestInfo import ClientOrderRequestInfo

from .StrategyInfo import StrategyInfo



class ClientOrderRequest(ClientOrderRequestInfo, metaclass=EzxMsgMeta):

	__block_fields__ = {
		'STGBLK' : Field('strategyInfo', 'STGBLK', noop_decode_func, ), 
	}

	def __init__(self, **kwargs):
		self.strategyInfo : StrategyInfo = None
		super(ClientOrderRequest, self).__init__(**kwargs)
		self.msg_type = api.IserverMsgType.APPLICATION.value
		self.msg_subtype = api.IserverMsgSubType.CLIENT_ORDER_REQUEST.value

