'''
 Generated by EZX Code Generator  
Date: 04/18/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api

from .SnapshotMdEntries import SnapshotMdEntries



class MarketDataSnapshot(EzxMsg, metaclass=EzxMsgMeta):

	__block_fields__ = {
		'SNAPSHOTMDENTRIESBLOCK' : Field('noMdEntires', 'SNAPSHOTMDENTRIESBLOCK', noop_decode_func, update_list_item), 
	}

	def __init__(self, **kwargs):
		self.fixEngineName : str = None
		self.mdMsgID : str = None
		self.symbol : str = None
		self.engineSeqno : int = None
		self.sendingTime : str = None
		self.msgRecvTime : str = None
		self.mdReqID : str = None
		self.text : str = None
		self.noMdEntires : list[SnapshotMdEntries] = None
		super(MarketDataSnapshot, self).__init__(**kwargs)
		self.msg_type = api.IserverMsgType.APPLICATION.value
		self.msg_subtype = api.IserverMsgSubType.MARKETDATASNAPSHOT.value

