'''
 Generated by EZX Code Generator  
Date: 04/18/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api




class IncrementalMdEntries(EzxMsg, metaclass=EzxMsgMeta):


	def __init__(self, **kwargs):
		self.mdentryRefID : str = None
		self.symbol : str = None
		self.mdentryType : str = None
		self.mdentryPrice : float = None
		self.msgRecvTime : str = None
		self.mdentrySize : float = None
		self.mdentryPositionNumber : int = None
		self.mdentryOriginator : str = None
		self.quoteCondition : str = None
		self.quoteEntryID : str = None
		self.ezxMdId : str = None
		self.mdUpdateAction : str = None
		super(IncrementalMdEntries, self).__init__(**kwargs)

