'''
 Generated by EZX Code Generator  
Date: 04/18/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api




class UpDown(EzxMsg, metaclass=EzxMsgMeta):


	def __init__(self, **kwargs):
		self.updownName : str = None
		self.decimalPlaces : int = None
		self.initialValue : float = None
		self.increment : float = None
		super(UpDown, self).__init__(**kwargs)

