'''
 Generated by EZX Code Generator  
Date: 04/18/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api




class OrderRequestStatus(EzxMsg, metaclass=EzxMsgMeta):


	def __init__(self, **kwargs):
		self.version : int = None
		self.reqState : str = None
		self.myID : str = None
		self.basketID : str = None
		self.clientOrderID : str = None
		self.importFileName : str = None
		self.importTimeStamp : str = None
		self.importJobName : str = None
		self.ezxClientOrderID : int = None
		self.ezxClientOrderVersion : int = None
		super(OrderRequestStatus, self).__init__(**kwargs)

