'''
 Generated by EZX Code Generator  
Date: 04/18/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api

from .ChoiceItem import ChoiceItem



class ChoiceGroup(EzxMsg, metaclass=EzxMsgMeta):

	__block_fields__ = {
		'CHOIBLK' : Field('choiceItems', 'CHOIBLK', noop_decode_func, update_list_item), 
	}

	def __init__(self, **kwargs):
		self.choiceGroupName : str = None
		self.choiceItems : list[ChoiceItem] = None
		super(ChoiceGroup, self).__init__(**kwargs)

