'''
 Generated by EZX Code Generator  
Date: 04/18/22 
'''


from iserver.EzxMsg import EzxMsg, EzxMsgMeta
from iserver.msgs import *
from iserver.enums import api




class LogonRequest(EzxMsg, metaclass=EzxMsgMeta):


	def __init__(self, **kwargs):
		self.companyName : str = None
		self.date : str = None
		self.dateGMT : str = None
		self.forceLogout : int = None
		self.logonType : int = None
		self.password : str = None
		self.seqNo : int = None
		self.userName : str = None
		self.myID : str = None
		self.sendDestStrategies : int = None
		self.sendSettings : int = None
		self.sendDestinationState : int = None
		self.omsVersion : str = None
		self.sendPosition : int = None
		self.creditUsageUpdateSecs : int = None
		super(LogonRequest, self).__init__(**kwargs)
		self.msg_type = api.IserverMsgType.APPLICATION.value
		self.msg_subtype = api.IserverMsgSubType.LOGON.value

